//============================================================================
// CFA-10006-12864B Demonstration code
// UTILS.C: General utility routines
// Samsung KS0107+KS0108 (S6B0107+S6B0108) LCD Controller
// Atmel ATMega32 processor @ 16MHz
// WinAVR / AVR GCC + Atmel AVR Studio
// Written by Brent A. Crosby
// Copyright 2005, Crystalfontz America, Inc. http://www.crystalfontz.com
//============================================================================
#include <avr/io.h>
#include "utils.h"
#include "timer.h"
#include "logo_screen.h"
#include "low_lcd.h"


//============================================================================
// Port Direction Initialization.
//
// This routine will set the default processor port directions.
//----------------------------------------------------------------------------
void Initialize_GPIO(void)
{
/* Port A - Bit 7: LCD_DB7,    			output, low
 *              6: LCD_DB6,             output, low
 *              5: LCD_DB5,        		output, low
 *              4: LCD_DB4,          	output, low
 *              3: LCD_DB3,             output, low
 *              2: LCD_DB2,             output, low
 *              1: LCD_DB1,             output, low
 *              0: LCD_DB0,             output, low
 */
     DDRA =  0b11111111;
     PORTA = 0b00000000;



/* Port B - Bit 7: LCD_CONTRAST_PWM,    output, low
 *              6: NC,                  input, pull-up
 *              5: LCD_BL_RED/K,        output, high
 *              4: LCD_BL_BLU,          output, high
 *              3: MISO,                output, low
 *              2: MOSI,                output, low
 *              1: SCK,                 output, low
 *              0: nSS,                 output, low
 */
     DDRB =  0b10111111;
     PORTB = 0b01110000;


/* Port C - Bit 7: LCD_DI,              output, high
 *              6: LCD_RW,              output, high
 *              5: LCD_E,               output, low
 *              4: LCD_CS1,             output, high
 *              3: LCD_CS2,             output, high
 *              2: LCD_RST,             output, low
 *              1: LCD_CS3,             output, high
 *              0: NC                   input, pull-up
 */
     DDRC =  0b11111110;
     PORTC = 0b11011011;

/* Port D - Bit 7: KEY0,                input, pull-up
 *              6: KEY1,                input, pull-up
 *              5: KEY2,                input, pull-up
 *              4: KEY3,                input, pull-up
 *              3: NC,                  input, pull-up
 *              2: LED2,                input, pull-up
 *              1: LED1,                input, pull-up
 *              0: NC,                  input, pull-up
 */
     DDRD =  0b00000110;
     PORTD = 0b11111001;

/* Port E - Bit 7: NC,                  input, pull-up
 *              6: NC,                  input, pull-up
 *              5: NC,                  input, pull-up
 *              4: NC,                  input, pull-up
 *              3: LCD_BL_GRN,          output, high
 *              2: NC,                  input, pull-up
 *              1: NC,                  input, pull-up
 *              0: NC,                  input, pull-up
 */
     DDRE =  0b00001000;
     PORTE = 0b11111111;

/* Port G - Bit 7: NC,                  input, pull-up
 *              6: NC,                  input, pull-up
 *              5: NC,                  input, pull-up
 *              4: LCD_EL,              output, high
 *              3: NC,                  input, pull-up
 *              2: NC,                  input, pull-up
 *              1: NC,                  input, pull-up
 *              0: NC,                  input, pull-up
 */
     DDRG =  0b00010000;
     PORTG = 0b11111111;
}
//============================================================================
void update_contrast(unsigned char contrast)
{
     OCR0A = ((unsigned short)contrast * 255) / 100;
//     OCR0A = ((uword)contrast * 128) / 100;
}
//============================================================================
void update_backlight(unsigned char backlight)
{
     OCR1A = ((unsigned short)backlight * 255) / 100;
//     OCR2A = ((uword)backlight * 255) / 100;
//    OCR3A = ((uword)backlight * 255) / 100;
}
//============================================================================
void Initialize_contrast_backlight_and_LCD()
{
	register unsigned char contrast;
	register unsigned char backlight;

    // copy the bitmap from flash to memory
	memcpy_P(display, boot_logo, sizeof(display));

	// Reset the LCD
	ResetLCD();

	//Start the timers.
	Initialize_And_Start_Timer_0_LCD_Contrast_PWM();
	Initialize_And_Start_Timer_1_32_Hz_and_Backlight();

	// Initializes the LCD, also takes LCD out of reset
	InitializeLCD();

	// Copies from the array in flash to RAM
    memcpy_P(display, boot_logo, sizeof(display));

    contrast = 20;
    update_contrast(contrast);
    backlight = 0;
    update_backlight(backlight);

}
